@@ -28,9 +28,18 @@ Rcpp::DataFrame qts2avts_impl(
 );
 
 // [[Rcpp::export]]
-Rcpp::DataFrame qts2aats_impl(
-    const Rcpp::DataFrame &qts
+Rcpp::DataFrame qts2aats_impl(const Rcpp::DataFrame &qts);
+
+// [[Rcpp::export]]
+Rcpp::DataFrame qts2rpyts_impl(const Rcpp::DataFrame &qts);
+
+// [[Rcpp::export]]
+Rcpp::DataFrame rpyts2qts_impl(const Rcpp::DataFrame &rpyts);
+
+// https://en.wikipedia.org/wiki/Conversion_between_quaternions_and_Euler_angles
+void GetRPYAngles(
+    const double &w, const double &x, const double &y, const double &z,
+    double &roll, double &pitch, double &yaw
 );
 
-
 #endif /* SQUATQTSTRANSFORMATIONS_H */
